import { DateComponent, ViewProps, Hit, DateRange, DateMarker, DateProfile, DateEnv, BaseOptionsRefined, DateProfileGenerator, Slicer, CoordRange, EventStore, EventUiHash, DateSpan, EventInteractionState, BaseComponent, EventRangeProps, CoordSpan, SegGroup } from '@fullcalendar/core/internal';
import { createElement, Ref } from '@fullcalendar/core/preact';
import { Duration } from '@fullcalendar/core';

interface TimelineViewState {
    clientWidth?: number;
    endScrollbarWidth?: number;
    slotInnerWidth?: number;
}
declare class TimelineView extends DateComponent<ViewProps, TimelineViewState> {
    private buildTimelineDateProfile;
    private computeSlotWidth;
    private headerScrollerRef;
    private bodyScrollerRef;
    private footerScrollerRef;
    private tDateProfile?;
    private bodyEl?;
    private slotWidth?;
    private headerRowInnerWidthMap;
    private bodySlotInnerWidth?;
    private syncedScroller;
    private scrollTime;
    render(): createElement.JSX.Element;
    componentDidMount(): void;
    componentDidUpdate(prevProps: ViewProps): void;
    componentWillUnmount(): void;
    handleBodySlotInnerWidth: (innerWidth: number) => void;
    handleSlotInnerWidths: () => void;
    handleClientWidth: (clientWidth: number) => void;
    handleEndScrollbarWidth: (endScrollbarWidth: number) => void;
    private updateSyncedScroller;
    private resetScroll;
    private handleTimeScrollRequest;
    private handleTimeScrollEnd;
    private applyTimeScroll;
    handeBodyEl: (el: HTMLElement | null) => void;
    queryHit(positionLeft: number, positionTop: number, elWidth: number, elHeight: number): Hit;
}

interface TimelineDateProfile {
    labelInterval: Duration;
    slotDuration: Duration;
    slotsPerLabel: number;
    headerFormats: any;
    isTimeScale: boolean;
    largeUnit: string;
    emphasizeWeeks: boolean;
    snapDuration: Duration;
    snapsPerSlot: number;
    normalizedRange: DateRange;
    timeWindowMs: number;
    slotDates: DateMarker[];
    isWeekStarts: boolean[];
    snapDiffToIndex: number[];
    snapIndexToDiff: number[];
    snapCnt: number;
    slotCnt: number;
    cellRows: TimelineHeaderCellData[][];
}
interface TimelineHeaderCellData {
    date: DateMarker;
    text: string;
    rowUnit: string;
    colspan: number;
    isWeekStart: boolean;
}
declare function buildTimelineDateProfile(dateProfile: DateProfile, dateEnv: DateEnv, allOptions: BaseOptionsRefined, dateProfileGenerator: DateProfileGenerator): TimelineDateProfile;

interface TimelineRange {
    startDate: DateMarker;
    endDate: DateMarker;
    isStart: boolean;
    isEnd: boolean;
}
type TimelineCoordRange = TimelineRange & CoordRange;
declare class TimelineLaneSlicer extends Slicer<TimelineRange, [
    DateProfile,
    DateProfileGenerator,
    TimelineDateProfile,
    DateEnv
]> {
    sliceRange(origRange: DateRange, dateProfile: DateProfile, dateProfileGenerator: DateProfileGenerator, tDateProfile: TimelineDateProfile, dateEnv: DateEnv): TimelineRange[];
}

interface TimelineLaneProps {
    dateProfile: DateProfile;
    tDateProfile: TimelineDateProfile;
    nowDate: DateMarker;
    todayRange: DateRange;
    nextDayThreshold: Duration;
    eventStore: EventStore | null;
    eventUiBases: EventUiHash;
    businessHours: EventStore | null;
    dateSelection: DateSpan | null;
    eventDrag: EventInteractionState | null;
    eventResize: EventInteractionState | null;
    eventSelection: string;
    resourceId?: string;
    slotWidth: number | undefined;
}
interface TimelineLaneState {
    segHeightRev?: string;
    moreLinkHeightRev?: string;
}
declare class TimelineLane extends BaseComponent<TimelineLaneProps, TimelineLaneState> {
    private sortEventSegs;
    private segHeightRefMap;
    private moreLinkHeightRefMap;
    private slicer;
    render(): createElement.JSX.Element;
    renderFgSegs(segs: (TimelineRange & EventRangeProps)[], segHorizontals: {
        [instanceId: string]: CoordSpan;
    }, segTops: Map<string, number>, forcedInvisibleMap: {
        [instanceId: string]: any;
    }, hiddenGroups: SegGroup<TimelineCoordRange>[], hiddenGroupTops: Map<string, number>, isDragging: boolean, isResizing: boolean, isDateSelecting: boolean): createElement.JSX.Element;
    private handleMoreLinkHeights;
    private handleSegHeights;
}

interface TimelineLaneBgProps {
    tDateProfile: TimelineDateProfile;
    nowDate: DateMarker;
    todayRange: DateRange;
    bgEventSegs: (TimelineRange & EventRangeProps)[] | null;
    businessHourSegs: (TimelineRange & EventRangeProps)[] | null;
    dateSelectionSegs: (TimelineRange & EventRangeProps)[];
    eventResizeSegs: (TimelineRange & EventRangeProps)[];
    slotWidth: number | undefined;
}
declare class TimelineLaneBg extends BaseComponent<TimelineLaneBgProps> {
    render(): createElement.JSX.Element;
    renderSegs(segs: (TimelineRange & EventRangeProps)[], fillType: string): createElement.JSX.Element;
}

interface TimelineSlatsProps {
    dateProfile: DateProfile;
    tDateProfile: TimelineDateProfile;
    nowDate: DateMarker;
    todayRange: DateRange;
    height?: number;
    slotWidth: number | undefined;
    innerWidthRef?: Ref<number>;
}
declare class TimelineSlats extends BaseComponent<TimelineSlatsProps> {
    private innerWidthRefMap;
    render(): createElement.JSX.Element;
    handleInnerWidths: () => void;
}

declare function createVerticalStyle(props: CoordSpan | undefined): {
    top: number;
    height: number;
} | undefined;
declare function createHorizontalStyle(// TODO: DRY up?
props: CoordSpan | undefined, isRtl: boolean): {
    left: number;
    width: number;
} | {
    right: number;
    width: number;
} | undefined;
declare function computeSlotWidth(slatCnt: number, slatsPerLabel: number, slatMinWidth: number | undefined, labelInnerWidth: number | undefined, viewportWidth: number | undefined): [
    canvasWidth: number | undefined,
    slatWidth: number | undefined,
    slatLiquid: boolean
];
declare function timeToCoord(// pixels
time: Duration, dateEnv: DateEnv, dateProfile: DateProfile, tDateProfile: TimelineDateProfile, slowWidth: number): number;

interface TimelineHeaderRowProps {
    dateProfile: DateProfile;
    tDateProfile: TimelineDateProfile;
    nowDate: DateMarker;
    todayRange: DateRange;
    rowLevel: number;
    isLastRow: boolean;
    cells: TimelineHeaderCellData[];
    innerHeighRef?: Ref<number>;
    innerWidthRef?: Ref<number>;
    slotWidth: number | undefined;
}
declare class TimelineHeaderRow extends BaseComponent<TimelineHeaderRowProps> {
    private innerWidthRefMap;
    private innerHeightRefMap;
    render(): createElement.JSX.Element;
    handleInnerWidths: () => void;
    handleInnerHeights: () => void;
    componentWillUnmount(): void;
}

interface TimelineNowIndicatorArrowProps {
    tDateProfile: TimelineDateProfile;
    nowDate: DateMarker;
    slotWidth: number | undefined;
}
declare class TimelineNowIndicatorArrow extends BaseComponent<TimelineNowIndicatorArrowProps> {
    render(): createElement.JSX.Element;
}

interface TimelineNowIndicatorLineProps {
    tDateProfile: TimelineDateProfile;
    nowDate: DateMarker;
    slotWidth: number | undefined;
}
declare class TimelineNowIndicatorLine extends BaseComponent<TimelineNowIndicatorLineProps> {
    render(): createElement.JSX.Element;
}

export { TimelineDateProfile, TimelineHeaderRow, TimelineLane, TimelineLaneBg, TimelineLaneProps, TimelineLaneSlicer, TimelineNowIndicatorArrow, TimelineNowIndicatorLine, TimelineRange, TimelineSlats, TimelineView, buildTimelineDateProfile, computeSlotWidth, createHorizontalStyle, createVerticalStyle, timeToCoord };
